@@ -19,13 +19,19 @@ creating the first AnyEvent watcher.
 
 package AnyEvent::Impl::EV;
 
-no warnings;
-use strict;
+use AnyEvent (); BEGIN { AnyEvent::common_sense }
+use EV 3.44;
 
-use EV;
+# cannot override directly, as EV doesn't allow arguments
+sub time       { EV::time       }
+sub now        { EV::now        }
+sub now_update { EV::now_update }
 
-sub time { EV::time }
-sub now  { EV::now  }
+*AE::time       = \&EV::time;
+*AE::now        = \&EV::now;
+*AE::now_update = \&EV::now_update;
+
+*AE::timer = \&EV::timer;
 
 sub timer {
    my ($class, %arg) = @_;
@@ -33,15 +39,17 @@ sub timer {
    EV::timer $arg{after}, $arg{interval}, $arg{cb}
 }
 
+*AE::io = defined &EV::_ae_io # 3.8
+   ? \&EV::_ae_io
+   : sub($$$) { EV::io $_[0], $_[1] ? EV::WRITE : EV::READ, $_[2] };
+
 sub io {
    my ($class, %arg) = @_;
 
-   my $cb = $arg{cb};
-
    EV::io
-      fileno $arg{fh},
+      $arg{fh},
       $arg{poll} eq "r" ? EV::READ : EV::WRITE,
-      $cb
+      $arg{cb}
 }
 
 sub signal {
@@ -50,6 +58,8 @@ sub signal {
    EV::signal $arg{signal}, $arg{cb}
 }
 
+*AE::signal = \&EV::signal;
+
 sub child {
    my ($class, %arg) = @_;
 
@@ -60,10 +70,22 @@ sub child {
    }
 }
 
+sub idle {
+   my ($class, %arg) = @_;
+
+   EV::idle $arg{cb}
+}
+
+*AE::idle = \&EV::idle;
+
 sub one_event {
    EV::loop EV::LOOP_ONESHOT;
 }
 
+sub loop {
+   EV::loop;
+}
+
 1;
 
 =head1 SEE ALSO
