@@ -9,6 +9,14 @@ AnyEvent::Impl::Perl - Pure-Perl event loop and AnyEvent adaptor for itself
   
    # this module gets loaded automatically as required
 
+   # Explicit use:
+   use AnyEvent::Impl::Perl;
+   use AnyEvent;
+
+   ...
+
+   AnyEvent::Impl::Perl::loop; # run the event loop
+
 =head1 DESCRIPTION
 
 This module provides transparent support for AnyEvent in case no other
@@ -32,8 +40,8 @@ This event loop has been optimised for the following use cases:
 =item monotonic clock is available
 
 This module will use the POSIX monotonic clock option (if it can be
-detected at runtime) or the POSIX C<times> function (if the resolution is
-better than 100Hz), in which case it will not suffer adversely from time
+detected at runtime) or the POSIX C<times> function (if the resolution
+is at least 100Hz), in which case it will not suffer adversely from time
 jumps.
 
 If no monotonic clock is available, this module will not attempt to
@@ -79,39 +87,51 @@ thousands of timers, though, or your timers have very small timeouts.
 
 =back
 
+=head1 FUNCTIONS
+
+The only user-visible function provided by this module is the C<loop>
+function:
+
+=over 4
+
+=item AnyEvent::Impl::Perl::loop
+
+Run the event loop, usually the last thing done in the main program when
+you want to use the pure-perl backend.
+
+=back
+
 =cut
 
 package AnyEvent::Impl::Perl;
 
-no warnings;
-use strict qw(vars subs);
-
 use Scalar::Util qw(weaken);
+use List::Util ();
 
-use AnyEvent ();
+use AnyEvent (); BEGIN { AnyEvent::common_sense }
 use AnyEvent::Util ();
 
-our $VERSION = 4.41;
+our $VERSION = $AnyEvent::VERSION;
 
 our ($NOW, $MNOW);
 
-sub MAXWAIT() { 1000 } # never sleep for longer than this many seconds
+sub MAXWAIT() { 3600 } # never sleep for longer than this many seconds
 
 BEGIN {
    local $SIG{__DIE__};
    my $time_hires = eval "use Time::HiRes (); 1";
-   my $clk_tck    = eval "use POSIX (); POSIX::sysconf (&POSIX::_SC_CLK_TCK)";
+   my $clk_tck    = eval "use POSIX (); POSIX::sysconf (POSIX::_SC_CLK_TCK ())";
    my $round; # actual granularity
 
-   if ($time_hires && eval "&Time::HiRes::clock_gettime (&Time::HiRes::CLOCK_MONOTONIC)") {
-      warn "AnyEvent::Impl::Perl using CLOCK_MONOTONIC as timebase.\n" if $AnyEvent::verbose >= 8;
+   if ($time_hires && eval "&Time::HiRes::clock_gettime (Time::HiRes::CLOCK_MONOTONIC ())") {
+      warn "AnyEvent::Impl::Perl: using CLOCK_MONOTONIC as timebase.\n" if $AnyEvent::VERBOSE >= 8;
       *_update_clock = sub {
          $NOW  = &Time::HiRes::time;
          $MNOW = Time::HiRes::clock_gettime (&Time::HiRes::CLOCK_MONOTONIC);
       };
 
-   } elsif (100 <= $clk_tck && $clk_tck <= 1000000 && "use POSIX (); (POSIX::times())[0] != -1") { # -1 is also a valid return value :/
-      warn "AnyEvent::Impl::Perl using POSIX::times (monotonic) as timebase.\n" if $AnyEvent::verbose >= 8;
+   } elsif (100 <= $clk_tck && $clk_tck <= 1000000 && eval { (POSIX::times ())[0] != -1 }) { # -1 is also a valid return value :/
+      warn "AnyEvent::Impl::Perl: using POSIX::times (monotonic) as timebase.\n" if $AnyEvent::VERBOSE >= 8;
       my $HZ1 = 1 / $clk_tck;
 
       my $last = (POSIX::times ())[0];
@@ -121,7 +141,7 @@ BEGIN {
 
          $next = (POSIX::times ())[0];
          # we assume 32 bit signed on wrap but 64 bit will never wrap
-         $last -= 0x100000000 if $last > $next;
+         $last -= 4294967296 if $last > $next; # 0x100000000, but perl has probelsm with big hex constants
          $MNOW += ($next - $last) * $HZ1;
          $last = $next;
       };
@@ -129,7 +149,7 @@ BEGIN {
       $round = $HZ1;
 
    } elsif (eval "use Time::HiRes (); 1") {
-      warn "AnyEvent::Impl::Perl using Time::HiRes::time (non-monotonic) clock as timebase.\n" if $AnyEvent::verbose >= 8;
+      warn "AnyEvent::Impl::Perl: using Time::HiRes::time (non-monotonic) clock as timebase.\n" if $AnyEvent::VERBOSE >= 8;
       *_update_clock = sub {
          $NOW = $MNOW = &Time::HiRes::time;
       };
@@ -148,6 +168,9 @@ _update_clock;
 sub now        { $NOW          }
 sub now_update { _update_clock }
 
+*AE::now        = sub () { $NOW };
+*AE::now_update = sub () { _update_clock };
+
 # fds[0] is for read, fds[1] is for write watchers
 # fds[poll][V] is the bitmask for select
 # fds[poll][W][fd] contains a list of i/o watchers
@@ -197,7 +220,7 @@ sub one_event {
 
       _update_clock;
 
-      if ($fds) {
+      if ($fds > 0) {
          # buggy microshit windows errornously sets exceptfds instead of writefds
          $vec[1] |= $vec[2] if AnyEvent::WIN32;
 
@@ -216,24 +239,29 @@ sub one_event {
                }
             }
          }
-      } elsif (AnyEvent::WIN32 && $! == AnyEvent::Util::WSAEINVAL) {
+      } elsif (AnyEvent::WIN32 && $fds && $! == AnyEvent::Util::WSAEINVAL) {
          # buggy microshit windoze asks us to route around it
          CORE::select undef, undef, undef, $wait if $wait;
-      } elsif (!@timer || $timer[0][0] > $MNOW) {
+      } elsif (!@timer || $timer[0][0] > $MNOW && !$fds) {
          $$$_ && $$$_->() for @idle = grep $$$_, @idle;
       }
    }
 }
 
-sub io {
-   my ($class, %arg) = @_;
+sub loop {
+   one_event while 1;
+}
 
-   my $fd = fileno $arg{fh};
+sub AE::io($$$) {
+   my ($fd, $write, $cb) = @_;
+
+   defined ($fd = fileno $fd)
+      or $fd = $_[0];
 
    my $self = bless [
       $fd,
-      $arg{poll} eq "w",
-      $arg{cb},
+      $write,
+      $cb,
       # q-idx
    ], "AnyEvent::Impl::Perl::io";
 
@@ -251,6 +279,12 @@ sub io {
    $self
 }
 
+sub io {
+   my (undef, %arg) = @_;
+
+   AE::io $arg{fh}, $arg{poll} eq "w", $arg{cb}
+}
+
 sub AnyEvent::Impl::Perl::io::DESTROY {
    my ($self) = @_;
 
@@ -273,24 +307,21 @@ sub AnyEvent::Impl::Perl::io::DESTROY {
    }
 }
 
-sub timer {
-   my ($class, %arg) = @_;
+sub AE::timer($$$) {
+   my ($after, $interval, $cb) = @_;
    
    my $self;
 
-   if ($arg{interval}) {
-      my $cb   = $arg{cb};
-      my $ival = $arg{interval};
-
-      $self = [$MNOW + $arg{after} , sub {
-         $_[0][0] = $MNOW + $ival;
+   if ($interval) {
+      $self = [$MNOW + $after , sub {
+         $_[0][0] = List::Util::max $_[0][0] + $interval, $MNOW;
          push @timer, $_[0];
          weaken $timer[-1];
          $need_sort = $_[0][0] if $_[0][0] < $need_sort;
          &$cb;
       }];
    } else {
-      $self = [$MNOW + $arg{after}, $arg{cb}];
+      $self = [$MNOW + $after, $cb];
    }
 
    push @timer, $self;
@@ -300,8 +331,14 @@ sub timer {
    $self
 }
 
+sub timer {
+   my (undef, %arg) = @_;
+
+   AE::timer $arg{after}, $arg{interval}, $arg{cb}
+}
+
 sub idle {
-   my ($class, %arg) = @_;
+   my (undef, %arg) = @_;
 
    push @idle, \\$arg{cb};
    weaken ${$idle[-1]};
